#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "zx4all.h"
#include "z80intrf.h"
#include "computer.h"
#include "menus.h"
#include "tape.h"

int elcontador=0;
int eltstado=0;
char elbit=0;

/* reads a tape file and updates the readed bit */

void tape_read(gzFile fichero, int tstados) {
	if(cmpt.tape_file_type == TAP_TAP)
		tape_read_tap(fichero,tstados);
	else
		tape_read_tzx(fichero,tstados);
	
	if(cmpt.pause)
		return;
	
}

// manages TAP files in REAL_MODE

void tape_read_tap (gzFile fichero, int tstados) {

	static unsigned char value, value2;
		
	if (fichero == NULL)
		return;

	if (!cmpt.pause) {
		if (cmpt.tape_current_mode == TAP_TRASH) {		// initialize a new block						
			gzread (fichero,&value, 1);
			gzread (fichero,&value2, 1);	// read block longitude
			if (gzeof (fichero)) {
				rewind_tape(fichero,1);
				cmpt.tape_current_mode = TAP_TRASH;
				return;
			}
			cmpt.tape_byte_counter = ((unsigned int) value) + 256 * ((unsigned int) value2);
			gzread (fichero,&(cmpt.tape_byte), 1);
			cmpt.tape_bit = 0x80;
			cmpt.tape_current_mode = TAP_GUIDE;
			cmpt.tape_counter0 = 2168;
			cmpt.tape_counter1 = 2168;
			if (!(0x80 & cmpt.tape_byte))
				cmpt.tape_counter_rep = 3228;	// 4 seconds
			else
				cmpt.tape_counter_rep = 1614;	// 2 seconds
		}

		// if there's a pulse still being reproduce, just reproduce it

		if (cmpt.tape_counter0) 	{
			if (cmpt.tape_counter0 > tstados) {
				cmpt.tape_readed = 0;	// return 0
				cmpt.tape_counter0 -= tstados;	// decrement counter;
				return;
			} else {
				tstados -= cmpt.tape_counter0;
				cmpt.tape_counter0 = 0;
			}
		}

		cmpt.tape_readed = 1;	// return 1
		if (cmpt.tape_counter1) {
			if (cmpt.tape_counter1 > tstados) {
				cmpt.tape_counter1 -= tstados;	// decrement counter;
				return;
			} else {
				tstados -= cmpt.tape_counter1;
				cmpt.tape_counter1 = 0;
				cmpt.tape_readed = 0;	// return 0
			}
		}

		// pulse ended
		
		switch (cmpt.tape_current_mode) {
		case TAP_GUIDE:	// guide tone
			if (cmpt.tape_counter_rep) {	// still into guide tone
				cmpt.tape_counter_rep--;
				cmpt.tape_counter0 = 2168 - tstados;
				cmpt.tape_counter1 = 2168;	// new pulse
				return;
			} else {	// guide tone ended. send sync tone
				cmpt.tape_counter0 = 667 - tstados;
				cmpt.tape_counter1 = 735;	// new pulse
				cmpt.tape_current_mode = TAP_DATA;	// data mode
				cmpt.tape_bit = 0x80;	// from bit0 to bit7
				return;
			}
			break;
		case TAP_DATA:	// data
			if (cmpt.tape_byte & cmpt.tape_bit) {	// next bit is 1
				cmpt.tape_counter0 = 1710 - tstados;
				cmpt.tape_counter1 = 1710;
			} else {
				cmpt.tape_counter0 = 851 - tstados;
				cmpt.tape_counter1 = 852;
			}
			cmpt.tape_bit = ((cmpt.tape_bit >> 1) & 0x7F);	// from bit0 to bit7
			if (!cmpt.tape_bit) {
				cmpt.tape_byte_counter--;
				if (!cmpt.tape_byte_counter) {	// ended the block
					cmpt.tape_current_mode = TAP_PAUSE;	// pause
					cmpt.tape_readed = 0;
					cmpt.tape_counter_rep = 3500000;	// 1 seconds
					return;
				}
				cmpt.tape_bit = 0x80;
				gzread (fichero,&(cmpt.tape_byte), 1);	// read next byte
				if (gzeof (fichero)) {
					rewind_tape(fichero,0);
					cmpt.tape_current_mode = TAP_STOP;	// stop tape
					return;
				}
			}
			break;
		case TAP_PAUSE:	// pause
			cmpt.tape_readed = 0;
			if (cmpt.tape_counter_rep > tstados) {
				cmpt.tape_counter_rep -= tstados;
			} else {
				cmpt.tape_counter_rep = 0;
				cmpt.tape_current_mode = TAP_TRASH;	// read new block
			}
			return;
			break;
		case TAP_STOP:
			cmpt.tape_current_mode = TAP_TRASH;	// initialize
			cmpt.pause = 1;	// pause it
			break;
		default:
			break;
		}
	}
}

// manages TZX files

void tape_read_tzx (gzFile fichero, int tstados) {

	static unsigned char value, value2,value3,value4,done;
	static unsigned int bucle,bucle2;
	
	if ((fichero == NULL)||(cmpt.pause))
		return;

	if (cmpt.tape_current_mode == TAP_TRASH) {		// initialize a new block
		done = 0;
		do {
			gzread(fichero,&value,1); // read block ID
// printf("%.2X\n",value); fflush(stdout);
			if(gzeof(fichero))
{
//puts("END"); fflush(stdout);
				done = 1;
}
			else
				switch(value) {
				case 0x10: // classic tape block
					done = 1;
					bucle = 0;
					cmpt.tape_current_bit = 0;
					cmpt.tape_bit0_level = 852;
					cmpt.tape_bit1_level = 1710;
					cmpt.tape_bits_at_end = 8;
					cmpt.tape_block_level = 2168;
					cmpt.tape_sync_level0 = 667;
					cmpt.tape_sync_level1 = 735;
					
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1); // pause length
					cmpt.tape_pause_at_end = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					if(cmpt.tape_pause_at_end==0)
						cmpt.tape_pause_at_end=10; // to avoid problems
					cmpt.tape_pause_at_end *= 3500;
					gzread (fichero,&value, 1);
					
					gzread (fichero,&value2, 1);	// read block longitude
					if (gzeof (fichero)) {
						rewind_tape(fichero,1);
						cmpt.tape_current_bit = 0;
						cmpt.tape_current_mode = TAP_TRASH;						
						for(bucle=0;bucle<10;bucle++)
							gzread(fichero,&value3,1); // jump over the header
						return;
					}
					cmpt.tape_byte_counter = ((unsigned int) value) + 256 * ((unsigned int) value2);
					gzread (fichero,&(cmpt.tape_byte), 1);
					cmpt.tape_bit = 0x80;
					cmpt.tape_current_mode = TAP_GUIDE;
					cmpt.tape_counter0 = 2168;
					cmpt.tape_counter1 = 2168;
					if (!(0x80 & cmpt.tape_byte))
						cmpt.tape_counter_rep = 3228;	// 4 seconds
					else
						cmpt.tape_counter_rep = 1614;	// 2 seconds					
					break;

				case 0x11: // turbo tape block					
					done = 1;
					bucle = 0;
					
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_block_level = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_sync_level0 = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_sync_level1 = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_bit0_level = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_bit1_level = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_counter_rep = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					cmpt.tape_counter_rep /=2;
					gzread(fichero,&value2,1);
					cmpt.tape_bits_at_end = value2;					
					
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1); // pause length
					cmpt.tape_pause_at_end = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					
			
					if(cmpt.tape_pause_at_end==0)
						cmpt.tape_pause_at_end=10; // to avoid problems
					cmpt.tape_pause_at_end *= 3500;
					
					gzread (fichero,&value, 1);
					gzread (fichero,&value2, 1);
					gzread (fichero,&value3, 1);	// read block longitude
					cmpt.tape_byte_counter = ((unsigned int) value) + 256 * ((unsigned int) value2) + 65536* ((unsigned int) value3);
					
					if (gzeof (fichero)) {
						rewind_tape(fichero,1);
						cmpt.tape_current_bit = 0;
						cmpt.tape_current_mode = TAP_TRASH;
						return;
					}
					
					gzread (fichero,&(cmpt.tape_byte), 1);
					cmpt.tape_bit = 0x80;
					cmpt.tape_current_mode = TAP_GUIDE;
					cmpt.tape_counter0 = cmpt.tape_block_level;
					cmpt.tape_counter1 = cmpt.tape_block_level;
					cmpt.tape_current_bit = 0;
					break;
					
				case 0x12: // pure tone
					done = 1;
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1); // length of pulse in T-states
					cmpt.tape_block_level = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					cmpt.tape_counter0 = cmpt.tape_block_level;
					cmpt.tape_counter1 = 0;
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1); // number of pulses
					cmpt.tape_counter_rep = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					if(cmpt.tape_counter_rep == 0)
						done = 0;
					cmpt.tape_current_mode = TZX_PURE_TONE;
					break;

				case 0x13: // multiple pulses
					done=1;
					gzread(fichero,&value2,1); // number of pulses
					cmpt.tape_counter_rep = ((unsigned int) value2);
					if(cmpt.tape_counter_rep == 0)
						done = 0;
					else {
						gzread(fichero,&value2,1);
						gzread(fichero,&value3,1); // length of pulse in T-states
						cmpt.tape_counter0 = ((unsigned int) value2) + 256 * ((unsigned int) value3);
						cmpt.tape_counter1 = 0;
						cmpt.tape_current_mode = TZX_SEQ_PULSES;
					}
					break;
				
				case 0x14: // turbo tape block					
					done = 1;
					bucle = 0;
					cmpt.tape_current_bit = 0;
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_bit0_level = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_bit1_level = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					gzread(fichero,&value2,1);
					cmpt.tape_bits_at_end = value2;
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1); // pause length
					cmpt.tape_pause_at_end = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					/*if(cmpt.tape_pause_at_end==0)
						cmpt.tape_pause_at_end=10;*/ // to avoid problems
					cmpt.tape_pause_at_end *= 3500;
					gzread (fichero,&value, 1);					
					gzread (fichero,&value2, 1);
					gzread (fichero,&value3,1);	// read block longitude
					if (gzeof (fichero)) {
						rewind_tape(fichero,1);
						cmpt.tape_current_bit = 0;
						cmpt.tape_current_mode = TAP_TRASH;						
						return;
					}
					cmpt.tape_byte_counter = ((unsigned int) value) + 256 * ((unsigned int) value2) + 65536*((unsigned int)value3);					
					cmpt.tape_bit = 0x80;
					gzread (fichero,&(cmpt.tape_byte), 1);	// read next byte
					if((cmpt.tape_byte_counter==1)&&(cmpt.tape_bits_at_end!=8)) { // last byte
						for(bucle=cmpt.tape_bits_at_end;bucle<8;bucle++) {
							cmpt.tape_byte=((cmpt.tape_byte>>1)&0x7F);
							cmpt.tape_bit = ((cmpt.tape_bit>>1)&0x7F);
						}
					}
					cmpt.tape_current_mode = TAP_DATA;
					cmpt.tape_counter0 = 0;
					cmpt.tape_counter1 = 0;
					cmpt.tape_counter_rep = 0;					
					break;

				case 0x20: // pause
					done = 1;
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1); // pause length
					cmpt.tape_counter_rep = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					cmpt.tape_readed = 0;
					cmpt.tape_counter0 = 0;				
					cmpt.tape_counter1 = 0; // 1ms of inversed pulse					
					if(cmpt.tape_counter_rep == 0) {
						cmpt.tape_current_mode = TAP_PAUSE2;	// initialize
						cmpt.tape_byte_counter = 31500;
						break;
					}
					cmpt.tape_counter_rep *= 3500;
					cmpt.tape_current_mode = TAP_PAUSE;									
					break;
					
				case 0x21: // group start
					gzread(fichero,&value2,1);
					bucle2=(unsigned int) value2;
					for(bucle=0;bucle<bucle2;bucle++)
						gzread(fichero,&value2,1);
					break;
					
				case 0x22: // group end
					break;
				
				case 0x24: // loop start
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					cmpt.tape_loop_counter = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					cmpt.tape_loop_pos = gztell(fichero);
					break;
				
				case 0x25: // loop end
					if(cmpt.tape_loop_counter) {
						cmpt.tape_loop_counter--;
						gzseek(fichero,cmpt.tape_loop_pos,SEEK_SET);
					}
					break;
				
				case 0x2A: // pause if 48K
					if(cmpt.mode128k==0) {
						cmpt.pause = 1;
						return;
					}
					break;
					
				case 0x30: // text description
					gzread(fichero,&value2,1); // length
					for(bucle=0;bucle<((unsigned int)value2);bucle++)
						gzread(fichero,&value3,1);
					break;
					
				case 0x31: // show text
					gzread(fichero,&value2,1);
					gzread(fichero,&value2,1); // length
					for(bucle=0;bucle<((unsigned int)value2);bucle++)
						gzread(fichero,&value3,1);
					break;
					
				case 0x32: // archive info
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1); // pause length
					bucle2 = ((unsigned int) value2) + 256 * ((unsigned int) value3);
					for(bucle=0;bucle<bucle2;bucle++)
						gzread(fichero,&value3,1);
					break;
				
				case 0x33: // hardware info
					gzread(fichero,&value2,1);					
					bucle2 = ((unsigned int) value2) *3;
					for(bucle=0;bucle<bucle2;bucle++)
						gzread(fichero,&value3,1);
					break;
					
				case 0x34: // emulation info					
					for(bucle=0;bucle<8;bucle++)
						gzread(fichero,&value3,1);
					break;
					
				case 0x35: // custon info					
					for(bucle=0;bucle<10;bucle++)
						gzread(fichero,&value3,1);
					gzread(fichero,&value,1);
					gzread(fichero,&value2,1);
					gzread(fichero,&value3,1);
					gzread(fichero,&value4,1);
					bucle2 = ((unsigned int) value) + 256 * ((unsigned int) value2) + 65536*((unsigned int) value3) + 16777216*((unsigned int) value4);
					for(bucle=0;bucle<bucle2;bucle++)
						gzread(fichero,&value3,1);
					break;
					
				default: // not supported
printf("---> Unsuported TZX. %X\n",value);
/*
					sprintf((char *)cmpt.osd_text,"Unsuported TZX. Contact FBZX autor. %X",value);
					cmpt.osd_time=200;
*/
					rewind_tape(fichero,1); // rewind and stop
					cmpt.tape_current_mode = TAP_TRASH;
					return;					
					break;
				}
		} while (!done);			
	}

	if (gzeof (fichero)) {
		rewind_tape(fichero,1);
		cmpt.tape_current_bit = 0;
		cmpt.tape_current_mode = TAP_TRASH;		
		return;
	}
	
	// if there's a pulse still being reproduce, just reproduce it

	if (cmpt.tape_counter0) 	{
		if (cmpt.tape_counter0 > tstados) {
			cmpt.tape_readed = cmpt.tape_current_bit;	// return current
			cmpt.tape_counter0 -= tstados;	// decrement counter;
			return;
		} else {
			tstados -= cmpt.tape_counter0;
			cmpt.tape_counter0 = 0;
		}
	}
	
	cmpt.tape_readed = 1 - cmpt.tape_current_bit;	// return oposite current
	if (cmpt.tape_counter1) {		
		if (cmpt.tape_counter1 > tstados) {
			cmpt.tape_counter1 -= tstados;	// decrement counter;
			return;
		} else {
			tstados -= cmpt.tape_counter1;
			cmpt.tape_counter1 = 0;
			cmpt.tape_readed = cmpt.tape_current_bit;	// return current
		}
	}

	// pulse ended

	switch (cmpt.tape_current_mode) {
	case TAP_FINAL_BIT:
		cmpt.tape_current_mode = TAP_TRASH;
		break;
	case TAP_GUIDE:	// guide tone
		if (cmpt.tape_counter_rep) {	// still into guide tone
			cmpt.tape_counter_rep--;
			cmpt.tape_counter0 = cmpt.tape_block_level - tstados;
			cmpt.tape_counter1 = cmpt.tape_block_level;	// new pulse
			return;
		} else {	// guide tone ended. send sync tone			
			cmpt.tape_counter0 = cmpt.tape_sync_level0 - tstados;
			cmpt.tape_counter1 = cmpt.tape_sync_level0;	// new pulse
			cmpt.tape_current_mode = TAP_DATA;	// data mode
			cmpt.tape_bit = 0x80;	// from bit0 to bit7
			if((cmpt.tape_byte_counter==1)&&(cmpt.tape_bits_at_end!=8)) { // last byte
				for(bucle=cmpt.tape_bits_at_end;bucle<8;bucle++) {
					cmpt.tape_byte=((cmpt.tape_byte>>1)&0x7F);
					cmpt.tape_bit = ((cmpt.tape_bit>>1)&0x7F);
				}
			}
			return;
		}
		break;
	case TAP_DATA:	// data
		if (cmpt.tape_byte & cmpt.tape_bit) {	// next bit is 1
			cmpt.tape_counter0 = cmpt.tape_bit1_level - tstados;
			cmpt.tape_counter1 = cmpt.tape_bit1_level;
		} else {
			cmpt.tape_counter0 = cmpt.tape_bit0_level - tstados;
			cmpt.tape_counter1 = cmpt.tape_bit0_level;
		}
		cmpt.tape_bit = ((cmpt.tape_bit >> 1) & 0x7F);	// from bit0 to bit7
		if (!cmpt.tape_bit) {			
			cmpt.tape_byte_counter--;
			if (!cmpt.tape_byte_counter) {	// ended the block
				if(cmpt.tape_pause_at_end) {					
					cmpt.tape_current_mode = TAP_PAUSE3;	// pause					
					cmpt.tape_counter_rep = cmpt.tape_pause_at_end;
				} else
					cmpt.tape_current_mode = TAP_FINAL_BIT;					
				return;
			}
			cmpt.tape_bit = 0x80;
			gzread (fichero,&(cmpt.tape_byte), 1);	// read next byte
			if (gzeof (fichero)) {
				rewind_tape (fichero,0);
				cmpt.tape_current_bit = 0;				
				cmpt.tape_current_mode = TAP_STOP;	// stop tape
				return;
			}
			if((cmpt.tape_byte_counter==1)&&(cmpt.tape_bits_at_end!=8)) { // last byte
				for(bucle=cmpt.tape_bits_at_end;bucle<8;bucle++) {
					cmpt.tape_byte=((cmpt.tape_byte>>1)&0x7F);
					cmpt.tape_bit = ((cmpt.tape_bit>>1)&0x7F);
				}
			}
		}
		break;
		
	case TAP_PAUSE3: // one pulse of 1 ms for ending the bit
		cmpt.tape_counter0 = 3500; // 1 ms
		cmpt.tape_counter1 = 0;		
		cmpt.tape_current_mode = TAP_PAUSE;
		break;
		
	case TAP_PAUSE:	// pause
		cmpt.tape_readed = 0;
		cmpt.tape_current_bit = 0;
		if (cmpt.tape_counter_rep > tstados) {
			cmpt.tape_counter_rep -= tstados;
		} else {
			cmpt.tape_counter_rep = 0;
			cmpt.tape_current_mode = TAP_TRASH;	// read new block
		}
		break;
	case TAP_PAUSE2:	// pause and stop
		cmpt.tape_readed = 0;
		cmpt.tape_current_bit = 0;
		if (cmpt.tape_counter_rep > tstados) {
			cmpt.tape_counter_rep -= tstados;
		} else {
			cmpt.tape_counter_rep = 0;
			cmpt.tape_current_mode = TAP_TRASH;	// read new block
			cmpt.pause = 1;
		}
		break;
	case TZX_PURE_TONE:
		cmpt.tape_counter_rep--;
		cmpt.tape_current_bit = 1 - cmpt.tape_current_bit; // invert current bit
		if (cmpt.tape_counter_rep) {	// still into guide tone			
			cmpt.tape_counter0 = cmpt.tape_block_level - tstados;
			cmpt.tape_counter1 = 0; // new pulse			
		} else
			cmpt.tape_current_mode = TAP_TRASH;	// next ID
		break;
	case TZX_SEQ_PULSES:
		cmpt.tape_current_bit = 1 - cmpt.tape_current_bit; // invert current bit
		cmpt.tape_counter_rep--;
		if(cmpt.tape_counter_rep) {
			gzread(fichero,&value2,1);
			gzread(fichero,&value3,1); // length of pulse in T-states
			cmpt.tape_counter0 = ((unsigned int) value2) + 256 * ((unsigned int) value3);
			cmpt.tape_counter1 = 0;
		} else
			cmpt.tape_current_mode = TAP_TRASH;	// next ID		
		break;
			
	case TAP_STOP:
		cmpt.tape_current_bit = 0;
		cmpt.tape_current_mode = TAP_TRASH;	// initialize
		cmpt.pause = 1;	// pause it
		break;
	default:
		break;
	}
}

void rewind_tape(gzFile fichero,unsigned char pause) {

	int thebucle;
	unsigned char value;
	
	gzrewind (cmpt.tap_file);
	if(cmpt.tape_file_type==TAP_TZX)
		for(thebucle=0;thebucle<10;thebucle++)
			gzread(cmpt.tap_file,&value,1); // jump over the header
	cmpt.pause=pause;
}

unsigned char file_empty(gzFile fichero) {
	
	long position,position2;
	
	position=gztell(fichero);
	gzseek(fichero,0,SEEK_END); // set the pointer at end
	position2=gztell(fichero);
	gzseek(fichero,position,SEEK_SET); // return the pointer to the original place
	if(position2==0)
		return 1; // empty file
	else
		return 0;	
}

void save_file(gzFile fichero) {

	long position;
	unsigned char x_or,salir;
	unsigned char dato;
	int longitud;
			
	position=gztell(fichero); // store current position
	gzseek(fichero,0,SEEK_END); // put position at end
	x_or=0;
	
	longitud=(int)_z80_get_DE();
	longitud+=2;
	
	dato=(unsigned char)(longitud%256);
	gzwrite(fichero,&dato,1);
	dato=(unsigned char)(longitud/256);
	gzwrite(fichero,&dato,1);

	dato=_z80_get_A();
	gzwrite(fichero,&dato,1);

	x_or^=_z80_get_A();

	salir = 0;
	do {
		if (_z80_get_DE() == 0)
			salir = 2;
		if (!salir) {
			dato=RdZ80(_z80_get_IX()); // read data
			gzwrite(fichero,&dato,1);
			x_or^=dato;
			_z80_set_IX(_z80_get_IX()+1);
		}
	} while (!salir);
	gzwrite(fichero,&x_or,1);
	_z80_set_IX(_z80_get_IX()+2);
	gzseek(fichero,position,SEEK_SET); // put position at end
	
	if(cmpt.tape_fast_load==1) //if we want fast load, we assume we want fast save too
		cmpt.other_ret = 1;	// next instruction must be RET
	
	return;
}

void fastload_block (gzFile fichero) {

	unsigned int longitud;
	unsigned char value[65536], salir,empty,flag_found;	
	unsigned int veces;

	cmpt.other_ret = 1;	// next instruction must be RET

	if (!(_z80_get_F() & C_FLAG)) { // if Carry=0, is VERIFY, so return OK
		_z80_set_F(_z80_get_F() | C_FLAG);	 // verify OK
		_z80_set_IX(_z80_get_IX() + _z80_get_DE());
		_z80_set_DE(0);
		return;
	}

	empty=file_empty(fichero);

	
	if ((fichero == NULL)||(empty)) {
		_z80_set_F(_z80_get_F() & (~C_FLAG));	// Load error
		_z80_set_IX(_z80_get_IX() + _z80_get_DE());
		_z80_set_DE(0);
/*
		if(empty)
			sprintf ((char *)cmpt.osd_text, "Tape file empty");
		else
			sprintf ((char *)cmpt.osd_text, "No tape selected");
		cmpt.osd_time = 100;
*/
		return;
	}

	veces=0;

	flag_found=0;
	do {
		gzread (fichero,value, 2);	// read length of current block
		if (gzeof (fichero))	{			// end of file?
			veces++; // one more time rewinded
/*
			sprintf ((char *)cmpt.osd_text, "Tape rewind");			
			cmpt.osd_time = 100;
*/
			gzrewind (fichero);	// again
			gzread (fichero,value, 2);	// read length of current block
		}
		longitud = ((unsigned int) value[0]) + 256 * ((unsigned int) value[1]);
		gzread (fichero,value, 1);	// read flag byte
		longitud--;
		if (value[0] != _z80_get_A()) { // different flag
			gzread (fichero,value, longitud);	// jump to the next block
			if (veces==3) { // tape rewinded three times? Block with that flag not found.
/*
				sprintf((char *)cmpt.osd_text,"Block with right flag not found");
				cmpt.osd_time = 100;
*/
				_z80_set_F(_z80_get_F() & (~C_FLAG));	// Load error
				return;
			}
		} else
			flag_found = 1;
	} while(flag_found == 0);

	salir = 0;
	do {
		if (longitud == 0)
			salir = 1;
		if (_z80_get_DE() == 0)
			salir = 2;
		if (!salir) {
			gzread (fichero,value, 1);	// read byte
			WrZ80 (_z80_get_IX(), (unsigned char) value[0]);	// store the byte
			_z80_set_IX(_z80_get_IX()+1);
			_z80_set_DE(_z80_get_DE()-1);
			longitud--;
		}
	}
	while (!salir);

	clean_screen ();

	if (salir == 1) { // system wants to load more bytes that the existent
		_z80_set_F(_z80_get_F() & (~C_FLAG));	// Load error
		return;
	}

	if ((salir == 2) && (longitud != 1)) { // there are more bytes to load
		_z80_set_F(_z80_get_F() & (~C_FLAG));	// Load error
		if (longitud > 1)
			gzread (fichero,value, longitud);	// jump to the next block
		return;
	}

	gzread (fichero,value, 1);	// jump over the checksum
	_z80_set_F(_z80_get_F() | C_FLAG);	// Load OK
	return;

}

static long init_pos=0;
static long end_pos=0;

int tape_select(char *fname)
{
	unsigned char char_id[16];

	if (!strncmp(fname,(char *)cmpt.current_tap,2048))
		return 1;
	if(cmpt.tap_file!=NULL)
		rewind_tape(cmpt.tap_file,1);
	cmpt.tape_current_bit=0;
	cmpt.tape_current_mode=TAP_TRASH;
	if(cmpt.tap_file!=NULL)
		gzclose(cmpt.tap_file);
	cmpt.tap_file=gzopen((const char *)fname,"rb");
	cmpt.tape_write = 0;
	if(cmpt.tap_file==NULL)
		return 0;
	strncpy((char *)cmpt.current_tap,(const char*)fname,2048);
	gzread(cmpt.tap_file,char_id,10);
	if((!strncmp((const char *)char_id,"ZXTape!",7)) && (char_id[7]==0x1A)&&(char_id[8]==1)) 
		cmpt.tape_file_type = TAP_TZX;
	else
		cmpt.tape_file_type = TAP_TAP;
	rewind_tape(cmpt.tap_file,1);
	cmpt.pause=1;
	cmpt.tape_fast_load=0;
	init_pos=gztell(cmpt.tap_file);
/*
	gzseek(cmpt.tap_file,0,SEEK_END);
	end_pos=gztell(cmpt.tap_file);
	gzseek(cmpt.tap_file,init_pos,SEEK_SET);
*/
init_pos=0;
end_pos=10;
	return 1;
}

long getpos_tape(gzFile fichero)
{
/*
long pos=cmpt.tape_counter0+cmpt.tape_counter1;
if (end_pos<pos)
	end_pos=pos;
printf("cnt0=%i, cnt1=%i, end=%i, pos=%i\n",cmpt.tape_counter0,cmpt.tape_counter1,end_pos,pos);
return end_pos-pos;
*/
/*
	long ret;
	if (!fichero) return 0;
	ret=gztell(fichero);
	if (ret<init_pos)
		return 0;
	if (ret>end_pos)
		return end_pos;
	return ret-init_pos;
*/ return 0;
}

long getpos_permil_tape(gzFile fichero)
{
/*
	long ret;
	double pos=(double)getpos_tape(fichero);
	pos*=1000.0;
	pos/=(double)end_pos;
	ret=pos;
	if (ret<0) ret=0;
	else if (ret>=1000)
		ret=999;
	return ret;
*/ return 0;
}
